@@ -4,6 +4,7 @@ import enum
 import json
 import os
 import ssl
+import sys
 import tempfile
 import traceback
 import warnings
@@ -137,6 +138,8 @@ class ClusterReport(object):
         The address of the scheduler, or None if not currently running.
     dashboard_link : str or None
         A link to the dashboard, or None if not currently running.
+    adaptive : bool
+        Whether the cluster is in adaptive mode.
     start_time : datetime.datetime
         The time the cluster was started.
     stop_time : datetime.datetime or None
@@ -155,6 +158,7 @@ class ClusterReport(object):
         "status",
         "scheduler_address",
         "dashboard_link",
+        "adaptive",
         "start_time",
         "stop_time",
         "tls_cert",
@@ -168,6 +172,7 @@ class ClusterReport(object):
         status,
         scheduler_address,
         dashboard_link,
+        adaptive,
         start_time,
         stop_time,
         tls_cert=None,
@@ -178,6 +183,7 @@ class ClusterReport(object):
         self.status = status
         self.scheduler_address = scheduler_address
         self.dashboard_link = dashboard_link
+        self.adaptive = adaptive
         self.start_time = start_time
         self.stop_time = stop_time
         self.tls_cert = tls_cert
@@ -196,7 +202,7 @@ class ClusterReport(object):
         )
 
     @classmethod
-    def _from_json(cls, gateway_address, proxy_address, msg):
+    def _from_json(cls, public_address, proxy_address, msg):
         start_time = datetime.fromtimestamp(msg.pop("start_time") / 1000)
         stop_time = msg.pop("stop_time")
         if stop_time is not None:
@@ -210,7 +216,7 @@ class ClusterReport(object):
 
         dashboard_route = msg.pop("dashboard_route")
         dashboard_link = (
-            "%s%s" % (gateway_address, dashboard_route) if dashboard_route else None
+            "%s%s" % (public_address, dashboard_route) if dashboard_route else None
         )
 
         return cls(
@@ -256,6 +262,12 @@ class Gateway(object):
             )
         address = address.rstrip("/")
 
+        public_address = format_template(dask.config.get("gateway.public-address"))
+        if public_address is None:
+            public_address = address
+        else:
+            public_address = public_address.rstrip("/")
+
         if proxy_address is None:
             proxy_address = format_template(dask.config.get("gateway.proxy-address"))
         if proxy_address is None:
@@ -271,6 +283,7 @@ class Gateway(object):
         proxy_address = "gateway://%s" % proxy_netloc
 
         self.address = address
+        self._public_address = public_address
         self.proxy_address = proxy_address
 
         self.auth = get_auth(auth)
@@ -320,7 +333,7 @@ class Gateway(object):
 
     def __del__(self):
         # __del__ is called even if __init__ fails, need to detect this
-        if hasattr(self, "_loop_runner"):
+        if hasattr(self, "_loop_runner") and not sys.is_finalizing():
             self.close()
 
     def __repr__(self):
@@ -375,7 +388,7 @@ class Gateway(object):
         req = HTTPRequest(url=url)
         resp = await self._fetch(req)
         return [
-            ClusterReport._from_json(self.address, self.proxy_address, r)
+            ClusterReport._from_json(self._public_address, self.proxy_address, r)
             for r in json.loads(resp.body).values()
         ]
 
@@ -396,6 +409,20 @@ class Gateway(object):
         """
         return self.sync(self._clusters, status=status, **kwargs)
 
+    def get_cluster(self, cluster_name, **kwargs):
+        """Get information about a specific cluster.
+
+        Parameters
+        ----------
+        cluster_name : str
+            The cluster name.
+
+        Returns
+        -------
+        report : ClusterReport
+        """
+        return self.sync(self._cluster_report, cluster_name, **kwargs)
+
     def _config_cluster_options(self):
         opts = dask.config.get("gateway.cluster.options")
         return {k: format_template(v) for k, v in opts.items()}
@@ -421,7 +448,7 @@ class Gateway(object):
 
         Returns
         -------
-        cluster_options : Options
+        cluster_options : dask_gateway.options.Options
             A dict of cluster options.
         """
         return self.sync(
@@ -459,7 +486,7 @@ class Gateway(object):
 
         Parameters
         ----------
-        cluster_options : Options, optional
+        cluster_options : dask_gateway.options.Options, optional
             An ``Options`` object describing the desired cluster configuration.
         **kwargs :
             Additional cluster configuration options. If ``cluster_options`` is
@@ -480,7 +507,7 @@ class Gateway(object):
         req = HTTPRequest(url=url)
         resp = await self._fetch(req)
         return ClusterReport._from_json(
-            self.address, self.proxy_address, json.loads(resp.body)
+            self._public_address, self.proxy_address, json.loads(resp.body)
         )
 
     async def _wait_for_start(self, cluster_name):
@@ -537,7 +564,7 @@ class Gateway(object):
 
         Parameters
         ----------
-        cluster_options : Options, optional
+        cluster_options : dask_gateway.options.Options, optional
             An ``Options`` object describing the desired cluster configuration.
         shutdown_on_close : bool, optional
             If True (default), the cluster will be automatically shutdown on
@@ -580,10 +607,10 @@ class Gateway(object):
         return self.sync(self._stop_cluster, cluster_name, **kwargs)
 
     async def _scale_cluster(self, cluster_name, n):
-        url = "%s/gateway/api/clusters/%s/workers" % (self.address, cluster_name)
+        url = "%s/gateway/api/clusters/%s/scale" % (self.address, cluster_name)
         req = HTTPRequest(
             url=url,
-            method="PUT",
+            method="POST",
             body=json.dumps({"worker_count": n}),
             headers=HTTPHeaders({"Content-type": "application/json"}),
         )
@@ -604,6 +631,44 @@ class Gateway(object):
         """
         return self.sync(self._scale_cluster, cluster_name, n, **kwargs)
 
+    async def _adapt_cluster(
+        self, cluster_name, minimum=None, maximum=None, active=True
+    ):
+        url = "%s/gateway/api/clusters/%s/adapt" % (self.address, cluster_name)
+        req = HTTPRequest(
+            url=url,
+            method="POST",
+            body=json.dumps({"minimum": minimum, "maximum": maximum, "active": active}),
+            headers=HTTPHeaders({"Content-type": "application/json"}),
+        )
+        await self._fetch(req)
+
+    def adapt_cluster(
+        self, cluster_name, minimum=None, maximum=None, active=True, **kwargs
+    ):
+        """Configure adaptive scaling for a cluster.
+
+        Parameters
+        ----------
+        cluster_name : str
+            The cluster name.
+        minimum : int, optional
+            The minimum number of workers to scale to. Defaults to 0.
+        maximum : int, optional
+            The maximum number of workers to scale to. Defaults to infinity.
+        active : bool, optional
+            If ``True`` (default), adaptive scaling is activated. Set to
+            ``False`` to deactivate adaptive scaling.
+        """
+        return self.sync(
+            self._adapt_cluster,
+            cluster_name,
+            minimum=minimum,
+            maximum=maximum,
+            active=active,
+            **kwargs,
+        )
+
 
 _widget_status_template = """
 <div>
@@ -896,6 +961,8 @@ class GatewayCluster(object):
         """
         if self.asynchronous:
             return self._stop_internal(shutdown=shutdown)
+        if self.status == "closed":
+            return
         if self.loop.asyncio_loop.is_running():
             self.gateway.sync(self._stop_internal, shutdown=shutdown)
         self.gateway.close()
@@ -917,7 +984,8 @@ class GatewayCluster(object):
         if self.asynchronous:
             # No del for async mode
             return
-        self.close()
+        if not sys.is_finalizing():
+            self.close()
 
     def __repr__(self):
         return "GatewayCluster<%s, status=%s>" % (self.name, self.status)
@@ -950,6 +1018,23 @@ class GatewayCluster(object):
         """
         return self.gateway.scale_cluster(self.name, n, **kwargs)
 
+    def adapt(self, minimum=None, maximum=None, active=True, **kwargs):
+        """Configure adaptive scaling for the cluster.
+
+        Parameters
+        ----------
+        minimum : int, optional
+            The minimum number of workers to scale to. Defaults to 0.
+        maximum : int, optional
+            The maximum number of workers to scale to. Defaults to infinity.
+        active : bool, optional
+            If ``True`` (default), adaptive scaling is activated. Set to
+            ``False`` to deactivate adaptive scaling.
+        """
+        return self.gateway.adapt_cluster(
+            self.name, minimum=minimum, maximum=maximum, active=active, **kwargs
+        )
+
     async def _watch_worker_status(self, comm):
         # We don't want to hold on to a ref to self, otherwise this will
         # leave a dangling reference and prevent garbage collection.
@@ -1002,7 +1087,7 @@ class GatewayCluster(object):
             return None
 
         try:
-            from ipywidgets import Layout, VBox, HBox, IntText, Button, HTML
+            from ipywidgets import Layout, VBox, HBox, IntText, Button, HTML, Accordion
         except ImportError:
             self._cached_widget = None
             return None
@@ -1016,12 +1101,30 @@ class GatewayCluster(object):
         request = IntText(0, description="Workers", layout=layout)
         scale = Button(description="Scale", layout=layout)
 
+        minimum = IntText(0, description="Minimum", layout=layout)
+        maximum = IntText(0, description="Maximum", layout=layout)
+        adapt = Button(description="Adapt", layout=layout)
+
+        accordion = Accordion(
+            [HBox([request, scale]), HBox([minimum, maximum, adapt])],
+            layout=Layout(min_width="500px"),
+        )
+        accordion.selected_index = None
+        accordion.set_title(0, "Manual Scaling")
+        accordion.set_title(1, "Adaptive Scaling")
+
         @scale.on_click
         def scale_cb(b):
             with log_errors():
                 self.scale(request.value)
 
-        elements = [title, HBox([status, request, scale])]
+        @adapt.on_click
+        def adapt_cb(b):
+            self.adapt(minimum=minimum.value, maximum=maximum.value)
+
+        name = HTML("<p><b>Name: </b>{0}</p>".format(self.name))
+
+        elements = [title, HBox([status, accordion]), name]
 
         if self.dashboard_link is not None:
             link = HTML(
